@@ -5,14 +5,14 @@ import (
 	"os"
 	"time"
 
-	kubemigratorclient "github.com/kubernetes-sigs/kube-storage-version-migrator/pkg/clients/clientset"
-	migrationv1alpha1informer "github.com/kubernetes-sigs/kube-storage-version-migrator/pkg/clients/informer"
 	configv1 "github.com/openshift/api/config/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
 	configv1client "github.com/openshift/client-go/config/clientset/versioned"
 	configinformers "github.com/openshift/client-go/config/informers/externalversions"
 	operatorv1client "github.com/openshift/client-go/operator/clientset/versioned"
 	operatorv1informers "github.com/openshift/client-go/operator/informers/externalversions"
+	operatorcontrolplaneclient "github.com/openshift/client-go/operatorcontrolplane/clientset/versioned"
+	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/connectivitycheckcontroller"
 	"github.com/openshift/library-go/pkg/controller/controllercmd"
 	"github.com/openshift/library-go/pkg/operator/encryption/controllers/migrators"
 	encryptiondeployer "github.com/openshift/library-go/pkg/operator/encryption/deployer"
@@ -22,30 +22,39 @@ import (
 	"github.com/openshift/library-go/pkg/operator/staticpod/controller/revision"
 	"github.com/openshift/library-go/pkg/operator/status"
 	"github.com/openshift/library-go/pkg/operator/v1helpers"
+	apiextensionsclient "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset"
+	apiextensionsinformers "k8s.io/apiextensions-apiserver/pkg/client/informers/externalversions"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/kubernetes"
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	apiregistrationclient "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset"
 	apiregistrationinformers "k8s.io/kube-aggregator/pkg/client/informers/externalversions"
 	utilpointer "k8s.io/utils/pointer"
+	kubemigratorclient "sigs.k8s.io/kube-storage-version-migrator/pkg/clients/clientset"
+	migrationv1alpha1informer "sigs.k8s.io/kube-storage-version-migrator/pkg/clients/informer"
 
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/apiservice"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/configobservation/configobservercontroller"
-	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/configobservation/etcdobserver"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/encryptionprovider"
-	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/oauthapiencryptioncontroller"
+	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/oauthapiencryption"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/operatorclient"
-	prune "github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/prunecontroller"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/resourcesynccontroller"
+	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/revisionpoddeployer"
 	"github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/v311_00_assets"
 	operatorworkload "github.com/openshift/cluster-openshift-apiserver-operator/pkg/operator/workload"
+	libgoassets "github.com/openshift/library-go/pkg/operator/apiserver/audit"
 	workloadcontroller "github.com/openshift/library-go/pkg/operator/apiserver/controller/workload"
 	apiservercontrollerset "github.com/openshift/library-go/pkg/operator/apiserver/controllerset"
+	libgoetcd "github.com/openshift/library-go/pkg/operator/configobserver/etcd"
+)
+
+const (
+	oauthAPIServerTargetNamespace = "openshift-oauth-apiserver"
 )
 
 func RunOperator(ctx context.Context, controllerConfig *controllercmd.ControllerContext) error {
@@ -65,6 +74,14 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 	if err != nil {
 		return err
 	}
+	operatorcontrolplaneClient, err := operatorcontrolplaneclient.NewForConfig(controllerConfig.KubeConfig)
+	if err != nil {
+		return err
+	}
+	apiextensionsClient, err := apiextensionsclient.NewForConfig(controllerConfig.KubeConfig)
+	if err != nil {
+		return err
+	}
 
 	operatorConfigInformers := operatorv1informers.NewSharedInformerFactory(operatorConfigClient, 10*time.Minute)
 	kubeInformersForNamespaces := v1helpers.NewKubeInformersForNamespaces(kubeClient,
@@ -73,8 +90,10 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		operatorclient.GlobalMachineSpecifiedConfigNamespace,
 		operatorclient.OperatorNamespace,
 		operatorclient.TargetNamespace,
-		etcdobserver.EtcdEndpointNamespace,
+		libgoetcd.EtcdEndpointNamespace,
 		metav1.NamespaceSystem,
+		"openshift-kube-apiserver",
+		oauthAPIServerTargetNamespace,
 	)
 	apiregistrationInformers := apiregistrationinformers.NewSharedInformerFactory(apiregistrationv1Client, 10*time.Minute)
 	configInformers := configinformers.NewSharedInformerFactory(configClient, 10*time.Minute)
@@ -106,18 +125,26 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 	}
 	versionRecorder.SetVersion("operator", os.Getenv("OPERATOR_IMAGE_VERSION"))
 
-	nodeProvider := encryptiondeployer.NewDeploymentNodeProvider(operatorclient.TargetNamespace, kubeInformersForNamespaces)
-	deployer, err := encryptiondeployer.NewRevisionLabelPodDeployer("revision", operatorclient.TargetNamespace, kubeInformersForNamespaces, resourceSyncController, kubeClient.CoreV1(), kubeClient.CoreV1(), nodeProvider)
+	openshiftNodeProvider := encryptiondeployer.NewDeploymentNodeProvider(operatorclient.TargetNamespace, kubeInformersForNamespaces)
+	openshiftDeployer, err := encryptiondeployer.NewRevisionLabelPodDeployer("revision", operatorclient.TargetNamespace, kubeInformersForNamespaces, resourceSyncController, kubeClient.CoreV1(), kubeClient.CoreV1(), openshiftNodeProvider)
 	if err != nil {
 		return err
 	}
+	oauthNodeProvider := encryptiondeployer.NewDeploymentNodeProvider(oauthAPIServerTargetNamespace, kubeInformersForNamespaces)
+	// note: do not pass resourceSyncController - CAO is responsible for synchronization
+	oauthDeployer, err := encryptiondeployer.NewRevisionLabelPodDeployer("revision", oauthAPIServerTargetNamespace, kubeInformersForNamespaces, nil, kubeClient.CoreV1(), kubeClient.CoreV1(), oauthNodeProvider)
+	if err != nil {
+		return err
+	}
+	weManageOauthConfig := encryptionprovider.IsOAuthEncryptionConfigManagedByThisOperator(kubeInformersForNamespaces.InformersFor(operatorclient.GlobalMachineSpecifiedConfigNamespace).Core().V1().Secrets().Lister().Secrets(operatorclient.GlobalMachineSpecifiedConfigNamespace), oauthAPIServerTargetNamespace, oauthapiencryption.EncryptionConfigManagedBy)
+	deployer, err := revisionpoddeployer.NewUnionDeployer(&revisionpoddeployer.AlwaysEnabledDeployer{Deployer: openshiftDeployer}, revisionpoddeployer.NewDisabledByPredicateDeployer(weManageOauthConfig, oauthDeployer))
 
 	migrationClient := kubemigratorclient.NewForConfigOrDie(controllerConfig.KubeConfig)
 	migrationInformer := migrationv1alpha1informer.NewSharedInformerFactory(migrationClient, time.Minute*30)
 	migrator := migrators.NewKubeStorageVersionMigrator(migrationClient, migrationInformer.Migration().V1alpha1(), kubeClient.Discovery())
 	encryptionProvider := encryptionprovider.New(
-		"openshift-oauth-apiserver",
-		oauthapiencryptioncontroller.EncryptionConfigManagedBy,
+		oauthAPIServerTargetNamespace,
+		oauthapiencryption.EncryptionConfigManagedBy,
 		[]schema.GroupResource{
 			{Group: "route.openshift.io", Resource: "routes"}, // routes can contain embedded TLS private keys
 			{Group: "oauth.openshift.io", Resource: "oauthaccesstokens"},
@@ -171,7 +198,8 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 				{Resource: "namespaces", Name: operatorclient.OperatorNamespace},
 				{Resource: "namespaces", Name: operatorclient.TargetNamespace},
 				{Resource: "namespaces", Name: "openshift-etcd-operator"}, // Capture events from etcd operator
-				{Resource: "endpoints", Name: etcdobserver.EtcdEndpointName, Namespace: etcdobserver.EtcdEndpointNamespace},
+				{Resource: "endpoints", Name: libgoetcd.EtcdEndpointName, Namespace: libgoetcd.EtcdEndpointNamespace},
+				{Group: "controlplane.operator.openshift.io", Resource: "podnetworkconnectivitychecks", Namespace: "openshift-apiserver"},
 			},
 			apiServicesReferences()...,
 		),
@@ -194,23 +222,30 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		configInformers.Config().V1().Images().Informer(),
 	).WithStaticResourcesController(
 		"APIServerStaticResources",
-		v311_00_assets.Asset,
+		libgoassets.WithAuditPolicies("audit", operatorclient.TargetNamespace, v311_00_assets.Asset),
 		[]string{
 			"v3.11.0/openshift-apiserver/ns.yaml",
 			"v3.11.0/openshift-apiserver/apiserver-clusterrolebinding.yaml",
 			"v3.11.0/openshift-apiserver/svc.yaml",
 			"v3.11.0/openshift-apiserver/sa.yaml",
 			"v3.11.0/openshift-apiserver/trusted_ca_cm.yaml",
+			libgoassets.AuditPoliciesConfigMapFileName,
 		},
 		kubeInformersForNamespaces,
 		kubeClient,
 	).WithRevisionController(
 		operatorclient.TargetNamespace,
-		nil,
-		[]revision.RevisionResource{{
-			Name:     "encryption-config",
-			Optional: true,
-		}},
+		[]revision.RevisionResource{
+			{
+				Name: "audit",
+			},
+		},
+		[]revision.RevisionResource{
+			{
+				Name:     "encryption-config",
+				Optional: true,
+			},
+		},
 		kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace),
 		OpenshiftDeploymentLatestRevisionClient{OperatorClient: operatorClient, TypedClient: operatorConfigClient.OperatorV1()},
 		v1helpers.CachedConfigMapGetter(kubeClient.CoreV1(), kubeInformersForNamespaces),
@@ -224,7 +259,14 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		configClient.ConfigV1().APIServers(),
 		configInformers.Config().V1().APIServers(),
 		kubeInformersForNamespaces,
-	).WithConfigUpgradableController().
+	).WithSecretRevisionPruneController(
+		operatorclient.TargetNamespace,
+		[]string{"encryption-config-"},
+		kubeClient.CoreV1(),
+		kubeClient.CoreV1(),
+		kubeInformersForNamespaces,
+	).
+		WithConfigUpgradableController().
 		WithLogLevelController()
 
 	runnableAPIServerControllers, err := apiServerControllers.PrepareRun()
@@ -232,22 +274,42 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		return err
 	}
 
+	oauthEncryptionController := oauthapiencryption.NewEncryptionConfigSyncController(
+		"OAuthAPIServerController",
+		oauthAPIServerTargetNamespace,
+		kubeClient.CoreV1(),
+		kubeInformersForNamespaces,
+		controllerConfig.EventRecorder,
+	)
+	auditPolicyPahGetter, err := libgoassets.NewAuditPolicyPathGetter("/var/run/configmaps/audit")
+	if err != nil {
+		return err
+	}
+
+	// TODO(4.7): switch over to secure access-token logging by default and delete old non-sha256 tokens
+	auditPolicyPathGetterWithAccessTokenLogs := func(profile string) (string, error) {
+		apiServerConfig, err := configInformers.Config().V1().APIServers().Lister().Get("cluster")
+		if errors.IsNotFound(err) {
+			return auditPolicyPahGetter(profile)
+		}
+		if err != nil {
+			return "", err
+		}
+
+		if apiServerConfig.Annotations["oauth-apiserver.openshift.io/secure-token-storage"] == "true" {
+			return auditPolicyPahGetter("secure-oauth-storage-" + profile)
+		}
+		return auditPolicyPahGetter(profile)
+	}
+
 	configObserver := configobservercontroller.NewConfigObserver(
 		kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace),
-		kubeInformersForNamespaces.InformersFor(etcdobserver.EtcdEndpointNamespace),
+		kubeInformersForNamespaces.InformersFor(libgoetcd.EtcdEndpointNamespace),
 		operatorClient,
 		resourceSyncController,
 		operatorConfigInformers,
 		configInformers,
-		controllerConfig.EventRecorder,
-	)
-
-	pruneController := prune.NewPruneController(
-		operatorclient.TargetNamespace,
-		[]string{"encryption-config-"},
-		kubeClient.CoreV1(),
-		kubeClient.CoreV1(),
-		kubeInformersForNamespaces,
+		auditPolicyPathGetterWithAccessTokenLogs,
 		controllerConfig.EventRecorder,
 	)
 
@@ -273,6 +335,18 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 		controllerConfig.Server.Handler.NonGoRestfulMux.Handle("/debug/controllers/resourcesync", debugHandler)
 	}
 
+	apiextensionsInformers := apiextensionsinformers.NewSharedInformerFactory(apiextensionsClient, 10*time.Minute)
+	connectivityCheckController := connectivitycheckcontroller.NewOpenshiftAPIServerConnectivityCheckController(
+		kubeClient,
+		operatorClient,
+		operatorcontrolplaneClient,
+		apiextensionsClient,
+		kubeInformersForNamespaces,
+		configInformers,
+		apiextensionsInformers,
+		controllerConfig.EventRecorder,
+	)
+
 	ensureDaemonSetCleanup(ctx, kubeClient, controllerConfig.EventRecorder)
 
 	operatorConfigInformers.Start(ctx.Done())
@@ -281,12 +355,14 @@ func RunOperator(ctx context.Context, controllerConfig *controllercmd.Controller
 	configInformers.Start(ctx.Done())
 	dynamicInformers.Start(ctx.Done())
 	migrationInformer.Start(ctx.Done())
+	apiextensionsInformers.Start(ctx.Done())
 
+	go oauthEncryptionController.Run(ctx, 1)
 	go configObserver.Run(ctx, 1)
 	go resourceSyncController.Run(ctx, 1)
-	go pruneController.Run(ctx, 1)
 	go runnableAPIServerControllers.Run(ctx)
 	go staleConditions.Run(ctx, 1)
+	go connectivityCheckController.Run(ctx, 1)
 
 	<-ctx.Done()
 	return nil
